/****************************************************************************
 *
 *                                 M U E S L I   v 1.3
 *
 *
 *     Copyright 2016 IMDEA Materials Institute, Getafe, Madrid, Spain
 *     Contact: muesli.materials@imdea.org
 *     Author: Ignacio Romero (ignacio.romero@imdea.org)
 *
 *     This file is part of MUESLI.
 *
 *     MUESLI is free software: you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation, either version 3 of the License, or
 *     (at your option) any later version.
 *
 *     MUESLI is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 *
 *     You should have received a copy of the GNU General Public License
 *     along with MUESLI.  If not, see <http://www.gnu.org/licenses/>.
 *
*****************************************************************************/







#pragma once
#ifndef _muesli_elastic_h
#define _muesli_elastic_h

#include "smallstrain.h"


namespace muesli
{
  class elasticMP;

  class elasticMaterial : public muesli::smallStrainMaterial
  {
  public:
                                elasticMaterial(const std::string& name,
                                                const materialProperties& cl);
                                elasticMaterial(const double E, const double nu,
                                                const double rho);
    virtual                     ~elasticMaterial(){}

    virtual bool                check() const;
    virtual muesli::smallStrainMP*  createMaterialPoint() const;
    virtual double              density() const;
    virtual double              getProperty(const propertyName p) const;
    virtual void                print(std::ostream &of=std::cout) const;
    virtual void                setRandom();
    virtual bool                test(std::ostream  &of=std::cout);
    virtual double              waveVelocity() const;

  private:
    double                      E, nu;              // Young's, Poisson
    double                      lambda, mu, bulk;   // Lame + bulk modulus
    double                      cp, cs;             // wave velocities
    double                      rho;                // density

#ifndef WEAKTHREADSAFE
      std::mutex                theMutex;
#endif
    friend class                elasticMP;
  };




  class elasticMP : public muesli::smallStrainMP
  {
  public:
                            elasticMP(const elasticMaterial &m);
    virtual                 ~elasticMP();
    virtual void            setRandom();


    // three dimensional response
    virtual void            contractWithDeviatoricTangent(const ivector &v1, const ivector &v2,itensor &T) const;
    virtual void            contractWithTangent(const ivector &v1, const ivector &v2,itensor &T) const;
    virtual void            tangentTensor(itensor4& C) const;
    virtual void            tangentMatrix(double C[6][6]) const;
    virtual double          plasticSlip() const;
    virtual double          volumetricStiffness() const;


    // energy
    virtual double          deviatoricEnergy() const;
    virtual double          dissipatedEnergy() const;
    virtual double          effectiveStoredEnergy() const;
    virtual double          storedEnergy() const;
    virtual double          volumetricEnergy() const;


    // stresses
    virtual double          pressure() const;
    virtual void            stress(istensor& sigma) const;
    virtual void            stressVector(double S[6]) const;
    virtual void            deviatoricStress(istensor& s) const;


    // uniaxial response
    virtual double          uniaxialStiffness() const;
    virtual double          uniaxialStress() const;

    virtual materialState   getConvergedState() const;
    virtual materialState   getCurrentState() const;


    // bookkeeping
    virtual void            commitCurrentState();
    virtual void            resetCurrentState();
    void                    setConvergedState(const double theTime, const istensor& strain);
    virtual void            updateCurrentState(const double t, const double strain11);
    virtual void            updateCurrentState(const double t, const istensor& strain);


  private:
    const elasticMaterial&  theElasticMaterial;
#ifndef WEAKTHREADSAFE
    std::mutex              theMutex;
#endif

  };
}

#endif
